@@ -3,23 +3,26 @@
     ## seq vector of observation indices
     v <- seq_len(n)
     ## number of groups
-    lev <- length(levels(control$strata))
-    ## compute nperms on number of levels
-    nperms <- numPerms(lev, control)
+    strata <- getStrata(control, which = "plots")
+    lev <- length(levels(strata))
+    ## compute nperms on number of levels - for this need Within()
+    ## and type == typeP
+    type <- getType(control, which = "plots")
+    newControl <- how(within = Within(type = type))
+    nperms <- numPerms(lev, newControl)
     ## result object
-    X <- matrix(nrow = nperms, ncol = length(control$strata))
+    X <- matrix(nrow = nperms, ncol = length(strata))
     ## store the type
-    type <- control$blocks$type
+    type <- getType(control, which = "plots")
+    mirror <- getMirror(control, which = "plots")
     perms <- if(type == "free") {
         allFree(lev)
     } else if(type == "series") {
-        mirror <- control$blocks$mirror
         allSeries(lev, nperms = nperms, mirror = mirror)
     } else if(type == "grid") {
-        nr <- control$blocks$nrow
-        nc <- control$blocks$ncol
-        mirror <- control$blocks$mirror
-        constant <- control$blocks$constant
+        nr <- getRow(control, which = "plots")
+        nc <- getCol(control, which = "plots")
+        constant <- getConstant(control)
         allGrid(lev, nperms = nperms, nr = nr, nc = nc,
                 mirror = mirror, constant = constant)
     } else {
@@ -28,10 +31,10 @@
         ## is possible given calling function...
         return(v)
     }
-    sp <- split(v, control$strata)
+    sp <- split(v, strata)
     ## build permutations by concatenating components of sp
     ## for each row of level permutations
     for(i in seq_len(nrow(perms)))
         X[i,] <- unname(do.call(c, sp[perms[i,]]))
-    return(X)
+    X
 }
